#define NUM_TABS 11


struct VertexToPixel
{
    float4 Position     : POSITION;
    float2 TexCoords    : TEXCOORD0;
};


struct PixelToFrame
{
    float4 Color : COLOR0;
};

Texture xImageTarget : ImageMap;
Texture xDepthTarget: DepthMap;
float2 xViewportSize: RenderTargetDimensions;
float xBrightness  : Brightness = 1.0f;



static const float gauss[NUM_TABS] = {
    0.3989,
    0.3814,
    0.3332,
    0.2661,
    0.1942,
    0.1295,
    0.0790,
    0.0440,
    0.0224,
    0.0104,
    0.0044
};

/*static const float gauss[NUM_TABS] = {
    0.3989,
    0.3332,
    0.1942,
    0.0790,
    0.0224,
    0.0044
};*/


float2 vMaxCoC = float2(30.0,60.0);


sampler ImageSampler = sampler_state { texture = <xImageTarget> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};
sampler DepthSampler = sampler_state { texture = <xDepthTarget> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=NONE; AddressU = mirror; AddressV = mirror;};


VertexToPixel ScreenQuadVertexShader( float4 inPos : POSITION, float4 inTexCoord : TEXCOORD0)
{
    VertexToPixel Output = (VertexToPixel)0;

    Output.Position = inPos;
    Output.TexCoords = inTexCoord;

    return Output;
}

PixelToFrame DOFRenderPixelShaderHor(VertexToPixel PSIn)
{
    PixelToFrame Output;

    float4 color;
    float discRadius;
    color = tex2D(DepthSampler, PSIn.TexCoords);
    discRadius = abs( color.r * vMaxCoC.y - vMaxCoC.x);
    color = 0;
    float fact = discRadius / (xViewportSize.x * (NUM_TABS - 1.0));
    for(float x = - NUM_TABS + 1.0; x < NUM_TABS; x++)
    {
    	float2 coord = float2(PSIn.TexCoords.x + x * fact, PSIn.TexCoords.y);
    	float4 tap = tex2D( ImageSampler, coord); 
    	float depth = tex2D(DepthSampler, coord).r;
    	tap.a = abs(depth * 2.0 -1.0) * gauss[abs(x)];
    	//tap.a = gauss[abs(x)];
    	color.rgb += tap.rgb * tap.a;
    	color.a += tap.a;	
    }
    Output.Color = color / color.a;
    Output.Color.a = 1.0;
    return Output;
}

PixelToFrame DOFRenderPixelShaderVer(VertexToPixel PSIn)
{
    PixelToFrame Output;

    float4 color;
    float discRadius;
    color = tex2D(DepthSampler, PSIn.TexCoords);
    discRadius = abs( color.r * vMaxCoC.y - vMaxCoC.x);
    color = 0;
    float fact = discRadius / (xViewportSize.y * (NUM_TABS - 1.0) );
    for(float y = - NUM_TABS + 1.0; y < NUM_TABS; y++)
    {
    	float2 coord = float2(PSIn.TexCoords.x, PSIn.TexCoords.y + y * fact);
    	float4 tap = tex2D( ImageSampler, coord); 
    	tap.a = tex2D(DepthSampler, coord).r;
    	tap.a = abs(tap.a * 2.0 -1.0) * gauss[abs(y)];
    	//tap.a = gauss[abs(y)];
    	color.rgb += tap.rgb * tap.a;
    	color.a += tap.a;	
    }
    Output.Color = color / color.a;
    Output.Color += xBrightness;
    
    Output.Color.a = 1.0;
    return Output;
}

technique DOF
{
    pass DOFRenderPassHor
    {
    	VertexShader = compile vs_3_0 ScreenQuadVertexShader();
    	Pixelshader = compile ps_3_0 DOFRenderPixelShaderHor();
    }
    pass DOFRenderPassVer
    {
    	VertexShader = compile vs_3_0 ScreenQuadVertexShader();
    	Pixelshader = compile ps_3_0 DOFRenderPixelShaderVer();
    }
}