struct VertexToPixel
{
    float4 Position     : POSITION;
    float2 TexCoords        : TEXCOORD0;
	  float3 depthBufferCoord: TEXCOORD1;
	  float reciprocalParticleSize: TEXCOORD2;
	  float4 color: TEXCOORD3;
	  float size : TEXCOORD4;
};

struct PixelToFrame
{
    float4 Color : COLOR0;
};

Texture xDiffuseTexture : DiffuseMap;
Texture xDepthTexture : DepthMap;
float4x4 xWorld : World;
float4x4 xView : View;
float4x4 xProjection : Projection;
float4x4 xWorldViewProj : WorldViewProjection;
float xParticleSize : ParticleSize = 1.0;

float xParticleRotation = 0.0f;

sampler TextureSampler = sampler_state { texture = <xDiffuseTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = mirror; AddressV = mirror;};
sampler DepthSampler = sampler_state { texture = <xDepthTexture> ; magfilter = LINEAR; minfilter = LINEAR; mipfilter=LINEAR; AddressU = mirror; AddressV = mirror;};

VertexToPixel ParticleVertexShader( float4 inPos : POSITION, float2 inTexCoord : TEXCOORD0, float3 inPartPos : TEXCOORD1, float4 inColor : TEXCOORD2, float inSize : TEXCOORD3 )
{

  VertexToPixel OUT;

	float3 viewNormal = -float3(xView[0].z, xView[1].z, xView[2].z);
	viewNormal = normalize(viewNormal);
	
	float3 particle_position = inPartPos.xyz;
	
	float4 eyespace = mul(float4(particle_position, 1.0), mul(xWorld, xView));
	float2 offset = float2(inPos.xy) * inSize;
	
	float3 worldPosition = mul(float4(particle_position, 1.0), xWorld).xyz;

	float angle_sin = sin(xParticleRotation);	
	float angle_cos = cos(xParticleRotation);
	
	float2x2 rotmatrix;
	rotmatrix[0][0] = angle_cos;
	rotmatrix[1][0] = -angle_sin;
	rotmatrix[0][1] = angle_sin;
	rotmatrix[1][1] = angle_cos;
	float2 rotate = mul(rotmatrix, offset);
	
	eyespace += float4(rotate.xy, -inSize, 0.0f);

  OUT.Position = mul(eyespace, xProjection);
    
  OUT.TexCoords = inTexCoord;
    
	OUT.depthBufferCoord.z = eyespace.z;
  float2 screen_coords = OUT.Position.xy / OUT.Position.w;
  screen_coords = screen_coords * 0.5f + 0.5f;
  screen_coords.y = 1.0f - screen_coords.y;
  OUT.depthBufferCoord.xy = screen_coords;	
    
  OUT.reciprocalParticleSize = 1 / inSize;
  OUT.color = inColor/256.0;
  OUT.size = inSize;
	return OUT;  
}

PixelToFrame ParticlePixelShader(VertexToPixel PSIn)
{
    PixelToFrame Output;
    float depth = tex2D(DepthSampler, PSIn.depthBufferCoord.xy).g;
    float difference = abs(depth - PSIn.depthBufferCoord.z);
		difference *= PSIn.reciprocalParticleSize;
		difference = min(difference, 1.0f);
    float4 col = tex2D(TextureSampler, PSIn.TexCoords);
    col.a *= difference;
    Output.Color = col * PSIn.color;
    return Output;
}

technique Particle
{
    pass Pass0
    {        
        VertexShader = compile vs_3_0 ParticleVertexShader();
        PixelShader = compile ps_3_0 ParticlePixelShader();
    }
}